/** crcpeek.c **********************************************************
 * Utility to view the checksum on a character file
 * Written 10/8/98 by Jumping Spider, released to the public domain.
 * Use of this program may constitute a violation of Thomas Biskup's
 * copyright on the Adom program.  This program is for educational
 * purposes only and the author assumes no liability for any such
 * usage.  Have a nice day.
 **********************************************************************/

/*----------------------------------------------------------------------
- Written using MS-DOS Edit 2.0.026, Copyright (c) 1995 Microsoft Corp.

- Compiled under MicroSoft Visual C++ 6.0.

- Adom uses the CRC-32 algorithm.  For details, consult any algorithm
    manual.  The general form of CRC is to take every byte of data and
    manipulate it as follows:

    bit(n) * x^(n) + bit(n-1) * x^(n-1) + ... + bit(0) * x^0

    The long & short of it is, this value is precomputed for each of the
    possible 256 byte values, and these values are stored in a table in
    the ADOM executable starting at 0x000C2982.  Actually, there are
    only 255 values, as the CRC of 0 is 0.

    This program overwrites those 4,092 bytes with zeroes.  This has the
    effect of causing the CRC-32 algorithm to always return a checksum
    of 0xFFFFFFFF.  All characters created after this patch will have
    this value as their checksum, and any previously created characters
    can be patched as well by simply overwriting the last 4-bytes of
    the savegame.
----------------------------------------------------------------------*/

#include "os.h"
#ifndef LINUX
#include <dos.h>
#endif
#include <stdio.h>
#include <stdlib.h>

int main(int _argc, char **_argv) {
  FILE            *fp;
  unsigned long   fubar;
  unsigned long   i;

  /* User didn't type a character file name */
  if (_argc != 2) {
    printf("CRCPeek 1.00.00\n");
    printf("Copyright (c) 1998 by Jumping Spider    dbrady@xmission.com\n");
    printf("Donated to the public domain.\n\n");
    printf("Usage:\n");
    printf("CRCPeek <filename>\n");
    printf("This utility reports the checksum of an ADOM 0.9.9 gamma 10+ savefile.\n");
    return(EXIT_SUCCESS);
  }

  /* Open character file for binary reading */
  if ((fp = fopen(_argv[1], "rb")) == NULL) {
    printf("Cannot open input file: %s.\n", _argv[1]);
    return(EXIT_FAILURE);
  }

  /* seek to EOF - 4 */
  if ((i = fseek(fp, -4, SEEK_END)) != 0) {
    printf("Could not seek to EOF-4.\n");
    return(EXIT_FAILURE);
  }

  /* read last long */
  i = fread(&fubar, sizeof(long), 1, fp);

  /* show result */
  if (fubar == 0x00000000)
    printf("This character has run out of luck!\n");
  else if (fubar == 0xFFFFFFFFL)
    printf("This character has been patched for editing.\n");
  else
    printf("This character has not been patched.  (CRC 0x%08X)\n", fubar);

  return(EXIT_SUCCESS);
}

