/** crcf.c *************************************************************
 * Program to fubar the CRC algorithm in Adom.exe, version 0.9.9g10.
 * Written 9/25/98 by Jumping Spider, released to the public domain.
 * Use of this program may constitute a violation of Thomas Biskup's
 * copyright on the Adom program.  This program is for educational
 * purposes only and the author assumes no liability for any such
 * usage.  Have a nice day.
 **********************************************************************/

/*----------------------------------------------------------------------
- Written using MS-DOS Edit 2.0.026, Copyright (c) 1995 Microsoft Corp.

- Compiled under MicroSoft Visual C++ 6.0.

- Adom uses the CRC-32 algorithm.  For details, consult any algorithm
    manual.  The general form of CRC is to take every byte of data and
    manipulate it as follows:

    bit(n) * x^(n) + bit(n-1) * x^(n-1) + ... + bit(0) * x^0

    The long & short of it is, this value is precomputed for each of the
    possible 256 byte values, and these values are stored in a table in
    the ADOM executable starting at 0x000C2982.  Actually, there are
    only 255 values, as the CRC of 0 is 0.

    This program overwrites those 4,092 bytes with zeroes.  This has the
    effect of causing the CRC-32 algorithm to always return a checksum
    of 0xFFFFFFFF.  All characters created after this patch will have
    this value as their checksum, and any previously created characters
    can be patched as well by simply overwriting the last 4-bytes of
    the savegame.
----------------------------------------------------------------------*/

#include "os.h"
#ifdef LINUX
#define ADOMEXE "adom"
#else
#define ADOMEXE "ADOM.EXE"
#endif

#include <stdio.h>
#include <stdlib.h>

int main(void)
{
    FILE            *fp;
    unsigned long   fubar;
    unsigned long   i,k;

    /* Open adom.exe for binary editing */
    if((fp = fopen(ADOMEXE, "rb+")) == NULL)
    {
        printf("Could not open %s.\n", ADOMEXE);
        return ( EXIT_FAILURE );
    }

    /* seek to the CRC lookup table */
    if((i = fseek(fp, 0x000C2982L, SEEK_SET)) != 0)
    {
        printf("Could not locate CRC table.\n");
        return ( EXIT_FAILURE );
    }

    /* stomp on the table by writing 255 0L's */
    fubar = 0;
    for(k = 0; k< 255; k++)
    {
        if((i = fwrite(&fubar, sizeof(long), 1, fp)) != 1)
        {
            printf("Couldn't write to CRC table.  Wrote %d bytes, expected 255.\n",k+1);
            return ( EXIT_FAILURE );
        }
    }

    printf("%s has been patched.  Checksums will now always be 0xFFFFFFFF.\n", ADOMEXE);
    return ( EXIT_SUCCESS );
}
